@@ -13,7 +13,9 @@ from dagster import (
     OutputDefinition,
     ResourceDefinition,
     String,
-    lambda_solid,
+    composite_solid,
+    fs_io_manager,
+    job,
     pipeline,
     repository,
     resource,
@@ -21,6 +23,7 @@ from dagster import (
 )
 from dagster.core.storage.file_manager import local_file_manager
 from dagster.utils import PICKLE_PROTOCOL, file_relative_path
+from dagstermill.io_managers import local_output_notebook_io_manager
 
 try:
     from dagster_pandas import DataFrame
@@ -62,13 +65,32 @@ def test_nb_solid(name, **kwargs):
     return dagstermill.define_dagstermill_solid(
         name=name,
         notebook_path=nb_test_path(name),
-        output_notebook="notebook",
+        output_notebook_name="notebook",
         output_defs=output_defs,
         **kwargs,
     )
 
 
-default_mode_defs = [ModeDefinition(resource_defs={"file_manager": local_file_manager})]
+def test_nb_op(name, path, **kwargs):
+    output_defs = kwargs.pop("output_defs", [OutputDefinition(is_required=False)])
+
+    return dagstermill.define_dagstermill_op(
+        name=name,
+        notebook_path=path,
+        output_notebook_name="notebook",
+        output_defs=output_defs,
+        **kwargs,
+    )
+
+
+default_mode_defs = [
+    ModeDefinition(
+        resource_defs={
+            "output_notebook_io_manager": local_output_notebook_io_manager,
+            "io_manager": fs_io_manager,
+        }
+    )
+]
 
 
 hello_world = test_nb_solid("hello_world", output_defs=[])
@@ -79,8 +101,31 @@ def hello_world_pipeline():
     hello_world()
 
 
-hello_world_with_custom_tags_and_description = test_nb_solid(
-    "hello_world", output_defs=[], tags={"foo": "bar"}, description="custom description"
+hello_world_op = test_nb_op(
+    "hello_world_op",
+    nb_test_path("hello_world"),
+    output_defs=[],
+)
+
+
+def build_hello_world_job():
+    @job(
+        resource_defs={
+            "output_notebook_io_manager": local_output_notebook_io_manager,
+        }
+    )
+    def hello_world_job():
+        hello_world_op()
+
+    return hello_world_job
+
+
+hello_world_with_custom_tags_and_description = dagstermill.define_dagstermill_solid(
+    name="hello_world_custom",
+    notebook_path=nb_test_path("hello_world"),
+    output_notebook_name="notebook",
+    tags={"foo": "bar"},
+    description="custom description",
 )
 
 
@@ -98,7 +143,7 @@ hello_world_config = test_nb_solid(
 goodbye_config = dagstermill.define_dagstermill_solid(
     name="goodbye_config",
     notebook_path=nb_test_path("print_dagstermill_context_solid_config"),
-    output_notebook="notebook",
+    output_notebook_name="notebook",
     config_schema={"farewell": Field(String, is_required=False, default_value="goodbye")},
 )
 
@@ -115,9 +160,9 @@ def alias_config_pipeline():
     goodbye_config.alias("aliased_goodbye")()
 
 
-@solid(input_defs=[InputDefinition("notebook", dagster_type=FileHandle)])
-def load_notebook(_, notebook):
-    return os.path.exists(notebook.path_desc)
+@solid(input_defs=[InputDefinition("notebook")])
+def load_notebook(notebook):
+    return notebook
 
 
 @pipeline(mode_defs=default_mode_defs)
@@ -126,6 +171,28 @@ def hello_world_with_output_notebook_pipeline():
     load_notebook(notebook)
 
 
+hello_world_no_output_notebook_no_file_manager = dagstermill.define_dagstermill_solid(
+    name="hello_world_no_output_notebook_no_file_manager",
+    notebook_path=nb_test_path("hello_world"),
+)
+
+
+@pipeline
+def hello_world_no_output_notebook_no_file_manager_pipeline():
+    hello_world_no_output_notebook_no_file_manager()
+
+
+hello_world_no_output_notebook = dagstermill.define_dagstermill_solid(
+    name="hello_world_no_output_notebook",
+    notebook_path=nb_test_path("hello_world"),
+)
+
+
+@pipeline(mode_defs=default_mode_defs)
+def hello_world_no_output_notebook_pipeline():
+    hello_world_no_output_notebook()
+
+
 hello_world_output = test_nb_solid("hello_world_output", output_defs=[OutputDefinition(str)])
 
 
@@ -172,21 +239,37 @@ mult_two_numbers = test_nb_solid(
 )
 
 
-@lambda_solid
+@solid
 def return_one():
     return 1
 
 
-@lambda_solid
+@solid
 def return_two():
     return 2
 
 
+@solid
+def return_three():
+    return 3
+
+
+@solid
+def return_four():
+    return 4
+
+
 @pipeline(mode_defs=default_mode_defs)
 def add_pipeline():
     add_two_numbers(return_one(), return_two())
 
 
+@pipeline(mode_defs=default_mode_defs)
+def double_add_pipeline():
+    add_two_numbers.alias("add_two_numbers_1")(return_one(), return_two())
+    add_two_numbers.alias("add_two_numbers_2")(return_three(), return_four())
+
+
 @solid(input_defs=[], config_schema=Int)
 def load_constant(context):
     return context.solid_config
@@ -300,12 +383,17 @@ def filepicklelist_resource(init_context):
             name="test",
             resource_defs={
                 "list": ResourceDefinition(lambda _: []),
-                "file_manager": local_file_manager,
+                "io_manager": fs_io_manager,
+                "output_notebook_io_manager": local_output_notebook_io_manager,
             },
         ),
         ModeDefinition(
             name="prod",
-            resource_defs={"list": filepicklelist_resource, "file_manager": local_file_manager},
+            resource_defs={
+                "list": filepicklelist_resource,
+                "output_notebook_io_manager": local_output_notebook_io_manager,
+                "io_manager": fs_io_manager,
+            },
         ),
     ]
 )
@@ -316,7 +404,11 @@ def resource_pipeline():
 @pipeline(
     mode_defs=[
         ModeDefinition(
-            resource_defs={"list": filepicklelist_resource, "file_manager": local_file_manager}
+            resource_defs={
+                "list": filepicklelist_resource,
+                "output_notebook_io_manager": local_output_notebook_io_manager,
+                "io_manager": fs_io_manager,
+            }
         )
     ]
 )
@@ -338,7 +430,7 @@ reimport = test_nb_solid(
 
 
 @solid
-def lister(_):
+def lister():
     return [1, 2, 3]
 
 
@@ -375,6 +467,99 @@ def failure_pipeline():
     test_nb_solid("yield_failure")()
 
 
+yield_something = test_nb_solid(
+    "yield_something",
+    input_defs=[InputDefinition("obj", str)],
+    output_defs=[OutputDefinition(str, "result")],
+)
+
+
+@solid
+def fan_in(a, b):
+    return f"{a} {b}"
+
+
+@pipeline(
+    mode_defs=[
+        ModeDefinition(
+            resource_defs={
+                "io_manager": fs_io_manager,
+                "output_notebook_io_manager": local_output_notebook_io_manager,
+            }
+        )
+    ]
+)
+def fan_in_notebook_pipeline():
+    a, _ = yield_something.alias("solid_1")()
+    b, _ = yield_something.alias("solid_2")()
+    fan_in(a, b)
+
+
+@pipeline(
+    mode_defs=[
+        ModeDefinition(
+            resource_defs={
+                "output_notebook_io_manager": local_output_notebook_io_manager,
+            }
+        )
+    ]
+)
+def fan_in_notebook_pipeline_in_mem():
+    a, _ = yield_something.alias("solid_1")()
+    b, _ = yield_something.alias("solid_2")()
+    fan_in(a, b)
+
+
+@composite_solid
+def outer():
+    yield_something()
+
+
+@pipeline(
+    mode_defs=[
+        ModeDefinition(
+            resource_defs={
+                "io_manager": fs_io_manager,
+                "output_notebook_io_manager": local_output_notebook_io_manager,
+            }
+        )
+    ]
+)
+def composite_pipeline():
+    outer()
+
+
+###################################################################################################
+# Back compat
+###################################################################################################
+
+hello_world_legacy = dagstermill.define_dagstermill_solid(
+    name="hello_world_legacy",
+    notebook_path=nb_test_path("hello_world"),
+    output_notebook="notebook",
+)
+
+
+@solid(input_defs=[InputDefinition("notebook", dagster_type=FileHandle)])
+def load_notebook_legacy(notebook):
+    return os.path.exists(notebook.path_desc)
+
+
+@pipeline(
+    mode_defs=[
+        ModeDefinition(
+            resource_defs={
+                "io_manager": fs_io_manager,
+                "file_manager": local_file_manager,
+            }
+        )
+    ]
+)
+def hello_world_with_output_notebook_pipeline_legacy():
+    notebook = hello_world_legacy()
+    load_notebook_legacy(notebook)
+
+
 @repository
 def notebook_repo():
     pipelines = [
@@ -385,6 +570,7 @@ def notebook_repo():
         hello_world_config_pipeline,
         hello_world_explicit_yield_pipeline,
         hello_world_output_pipeline,
+        hello_world_with_output_notebook_pipeline,
         hello_logging_pipeline,
         resource_pipeline,
         resource_with_exception_pipeline,
@@ -395,6 +581,10 @@ def notebook_repo():
         yield_obj_pipeline,
         retries_pipeline,
         failure_pipeline,
+        fan_in_notebook_pipeline_in_mem,
+        fan_in_notebook_pipeline,
+        hello_world_no_output_notebook_no_file_manager_pipeline,
+        hello_world_with_output_notebook_pipeline_legacy,
     ]
     if DAGSTER_PANDAS_PRESENT and SKLEARN_PRESENT and MATPLOTLIB_PRESENT:
         pipelines += [tutorial_pipeline]
